/** \file    basic_types.h
 *  \brief   basic types
 *  \author  Takahisa Toda
 */
#ifndef BASIC_TYPES_H
#define BASIC_TYPES_H

#include <stdio.h>
#include <assert.h>
#include "vec.h"

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/********THIS PART is COPIED FROM solver.h of MiniSat-C_v1.14.1************/
typedef int  bool;
static const bool  true      = 1;
static const bool  false     = 0;

typedef int                lit;
typedef char               lbool;

#ifdef _WIN32
typedef signed __int64     uint64;   // compatible with MS VS 6.0
#else
typedef unsigned long long uint64;
#endif

static const int   var_Undef = -1;
static const lit   lit_Undef = -2;

static const lbool l_Undef   =  0;
static const lbool l_True    =  1;
static const lbool l_False   = -1;

static inline lit  toLit   (int v) { return v + v; }
static inline lit  lit_neg (lit l) { return l ^ 1; }
static inline int  lit_var (lit l) { return l >> 1; }
static inline int  lit_sign(lit l) { return (l & 1); }
/*****************************************************************************/

typedef vecp dnf_t;

static inline void *vecp_pop(vecp *f)
{
    const int n = vecp_size(f);
    assert(n > 0);

    veci *t = vecp_begin(f)[n-1];
    vecp_resize(f, n-1);

    return t;
}

static inline void dnf_delete(dnf_t *f)
{
    const int m = vecp_size(f);
    for (int i = 0; i < m; i++) {
        veci *t = vecp_begin(f)[i];
        veci_delete(t);
        free(t);
    }
    vecp_delete(f);
}


// A part of this function was copied from solver.c of minisat
static inline void dnf_sort(dnf_t *f)
{
    const int m = vecp_size(f);
    for (int k = 0; k < m; k++) {
        veci *t    = vecp_begin(f)[k];
        lit  *begin = veci_begin(t);
        lit  *end   = veci_begin(t) + veci_size(t);
   
        lit *i; lit *j; 
        for (i = begin + 1; i < end; i++){
            lit l = *i;
            for (j = i; j > begin && *(j-1) > l; j--)
                *j = *(j-1);
            *j = l;
        }
    }
}


static inline void fprint_term(FILE *p, veci *t)
{
    const int n = veci_size(t);
    int *lits   = veci_begin(t);

    for (int i = 0; i < n; i++) {
        if (lit_sign(lits[i]))
            fprintf(p, "-");
        fprintf(p, "%d ", lit_var(lits[i])+1);
    }
    fprintf(p, "0\n");
}

static inline void print_term(veci *t)
{
    fprint_term(stdout, t);
}

#endif /*BASIC_TYPES_H*/
